import xler8
import sys
import csv
import copy
from openpyxl.utils import get_column_letter
import rich
import rich.console
import rich.table

infile = sys.argv[1]

C = rich.console.Console(width=8192)

t = rich.table.Table(title="file:%s" % (infile), safe_box=True, padding=0)


hdrs = xler8.headers0(infile)
data_ = xler8.xlsx0(infile)

t.add_column("net-row-1", no_wrap=True)
for h in hdrs:
    t.add_column(h, no_wrap=True)

for i in range(len(data_[hdrs[0]])):
    row2 = [str(i+1)] + [ " >"+str(x)+"< :%s " % type(x).__name__ for x in [data_[y][i] for y in hdrs]]
    t.add_row(*row2)

C.print(t)



import xler8
import datetime
import sys


def dmaker_text(days_back=7):
    res = []
    start_day = datetime.datetime.now(tz=datetime.UTC)
    some_days = [ start_day - datetime.timedelta(days=x) for x in range(days_back) ]
    for dt in some_days:
        dts = dt.isoformat().split("T")[0]
        dows = dt.strftime(" %a ")
        if dt.isoweekday() >= 6:
            dows = "     "
        res.append(dts + dows)
    res.reverse()
    for line in res:
        print(line)


if len(sys.argv) > 1:
    dmaker_text(int(sys.argv[1]))
else:
    dmaker_text()



import xler8
import sys
import csv
import copy
from openpyxl.utils import get_column_letter

infile = sys.argv[1]
colname = sys.argv[2]


hdrs = xler8.headers0(infile)
data = xler8.xlsx0(infile)
for i in range(len(data[colname])):
    print(data[colname][i])



import sys
import xler8
import copy


def diff_xlsx(in1_filename, in2_filename, out1_filename, compare_cols):
    in1 = xler8.xlsx(in1_filename)
    in1_sheets = xler8.xlsx_sheetnames(in1_filename)
    
    in2 = xler8.xlsx(in2_filename)
    in2_sheets = xler8.xlsx_sheetnames(in2_filename)
    
    sheet1 = in1_sheets[0]
    in1_hdr = list(in1[sheet1].keys())
    
    sheet2 = in2_sheets[0]
    in2_hdr = list(in1[sheet2].keys())

    compare_indices = []
    for col in compare_cols:
        compare_indices.append(in1_hdr.index(col))

    in1_rows = []
    in2_rows = []

    for r0 in range(0, len(in1[sheet1][in1_hdr[0]])):
        row = [ in1[sheet1][x][r0] for x in in1_hdr ]
        in1_rows.append(row)

    for r0 in range(0, len(in2[sheet2][in2_hdr[0]])):
        row = [ in2[sheet2][x][r0] for x in in2_hdr ]
        in2_rows.append(row)

    res_data = [in1_hdr]
    
    other = in2_rows
    for row in in1_rows:
        row_in_other = False
        for other_row in other:
            col_match_count = 0
            for col_index in compare_indices:
                if row[col_index] == other_row[col_index]:
                    col_match_count += 1
                
            if col_match_count == len(compare_indices):
                row_in_other=True
        
        if row_in_other == False:
            res_data.append(row)
    
    other = in1_rows
    for row in in2_rows:
        row_in_other = False
        for other_row in other:
            col_match_count = 0
            for col_index in compare_indices:
                if row[col_index] == other_row[col_index]:
                    col_match_count += 1
                
            if col_match_count == len(compare_indices):
                row_in_other=True

        if row_in_other == False:
            res_data.append(row)
    
    xler8.out(out1_filename, sheets={sheet1:{'data': res_data}})


arg_in1 = sys.argv[1]
arg_in2 = sys.argv[2]
arg_out1 = sys.argv[3]
arg_cols = sys.argv[4].split(",")

print("Trying to generate Diff [%s] <-> [%s] into [%s] based on comparing %d columns %s" % (arg_in1, arg_in2, arg_out1, len(arg_cols), str(arg_cols)))
diff_xlsx(arg_in1, arg_in2, arg_out1, arg_cols)



import xler8
import argparse
import copy
from pprint import pprint as PP

parser = argparse.ArgumentParser()
parser.add_argument("-i", nargs=1, type=str, metavar="XLSX-INPUT", help="")
parser.add_argument("-t", nargs=1, type=str, action="append", metavar="COLUMN-TITLE", help="")
parser.add_argument("-f", nargs=1, type=str, action="append", metavar="CONTENT-TEXT-FILE", help="")
parser.add_argument("-o", nargs=1, type=str, metavar="VBA-OUTPUT-FILE", help="")


args = parser.parse_args()

match_values = {}

for i in range(len(args.t)):
    content = None
    with open(args.f[i][0], 'r') as f:
        content = f.read().strip().split("\n")
    match_values[args.t[i][0]] = copy.deepcopy(content)


data = xler8.xlsx0(args.i[0])
data_headers = xler8.headers0(args.i[0])

all_the_reasons = {}


for row in range(len(data[data_headers[0]])):
    # row=0 => BCDEF....:2 !!!
    kill_row_index = row+2
    for h in match_values.keys():
        if data[h][row] in match_values[h]:
            textual_kill = "%08d" % kill_row_index
            if not textual_kill in all_the_reasons.keys():
                all_the_reasons[textual_kill] = []
            all_the_reasons[textual_kill].append("column %s matches %s" % (h, data[h][row]))

# padded row indice
ordered = list(reversed(list(sorted(all_the_reasons.keys()))))

with open(args.o[0], 'w') as f:
    f.write("' auto generated xler8.vba row-killer\n");
    f.write("\n");
    f.write("Sub ReverseRowKiller()\n");
    f.write("  With Sheet1\n");
    for row_ in ordered:
        row = int(row_)
        f.write("    .Rows(%d).EntireRow.Delete\n" % row);
    f.write("  End With\n");
    f.write("End Sub\n");
    f.write("\n");


report_data = [['Row Number', 'Reasons', 'Number of Reasons']]

for row_ in reversed(ordered):
    row = int(row_)
    reasons = ", ".join(all_the_reasons[row_])
    report_data.append([str(row), reasons, str(len(all_the_reasons[row_]))])


filename_report = args.i[0] + "-row-kill-report.xlsx"
xler8.xlsx_out(filename=filename_report, sheets={
    "Reasons": {
        'data': report_data,
        'cw': xler8.cw_gen(report_data)
    }
})


import xler8
import datetime
import sys


def work_overview(filename, daymins=480):
    lines = []
    with open(filename, 'r') as f:
        lines = [ x.strip().split(" ") for x in f.read().strip().split("\n") if x.strip() != ""]
    
    alltags = {}
    alldays = {}

    for linecols in lines:
        dt = linecols[0] # date
        tg = linecols[1] # tag
        tm = int(linecols[2]) # tag minutes
        alltags[tg]=True
        alldays[dt]=True

    alltags = list(sorted(alltags.keys()))
    alldays = list(sorted(alldays.keys()))
    start_day_s = alldays[0]
    latest_day_s = alldays[-1]

    start_day = datetime.datetime.fromisoformat(start_day_s + "T00:00:00+00:00")
    latest_day = datetime.datetime.fromisoformat(latest_day_s + "T00:00:00+00:00")
    data_days = int((latest_day-start_day).days)

    some_days = [ start_day + datetime.timedelta(days=x) for x in range(data_days+1) ]

    data_ = {}
    for dt in some_days:
        dts = dt.isoformat().split("T")[0]
        minutes_value = (-1)*daymins
        if dt.isoweekday() >= 6:
            minutes_value = 0
        data_[dts] = { 'date': dts, 'minutes': minutes_value, 'acc': [0 for x in range(len(alltags))]}


    # accumulate data_ with logged minutes
    for linecols in lines:
        dt = linecols[0] # date
        tg = linecols[1] # tag
        tm = int(linecols[2]) # tag minutes
        o = data_[dt]
        tag_idx = alltags.index(tg)
        o['minutes'] += tm
        o['acc'][tag_idx] += tm


    data = [["day-of-week", "date", "day-miss"] + alltags]

    for d_ in some_days:
        d=d_.isoformat().split("T")[0]
        wota = d_.strftime("%a")
        if d_.isoweekday() >= 6:
            wota = ""
        if d in data_.keys():
            data.append([wota, d, data_[d]['minutes']]+data_[d]['acc'])
        else:
            data.append([wota, d])

    xler8.xlsx_out("%s.xlsx" % filename, sheets={
        "work":{
            'data': data,
            'cw': xler8.cw_gen(data, 2)
        }
    })

    print(start_day_s)
    print(latest_day_s)
    print((latest_day-start_day).days)


work_overview(sys.argv[1])



import xler8
import sys
import csv
import copy
from openpyxl.utils import get_column_letter
import rich
import rich.console
import rich.table

infile = sys.argv[1]

o_del = ","
o_del = ";"
o_quote = '"'

try:
    o_del = sys.argv[2]
    o_quote = sys.argv[3]
except:
    pass

C = rich.console.Console(width=8192)

t = rich.table.Table(title="file:%s:d=%s:q=%s" % (infile, o_del, o_quote), safe_box=True, padding=0)

hdrs=[]
data = []

with open(infile, newline='') as csvfile:
    csvreader = csv.reader(csvfile, delimiter=o_del, quotechar=o_quote)

    h_mode=True
    for row in csvreader:
        if h_mode:
            h_mode = False
            hdrs = copy.deepcopy(row)
            continue

        row2 = [" >" + x + "< " for x in row]
        data.append(row2)

t.add_column("net-row-1", no_wrap=True)
for h in hdrs:
    t.add_column(h, no_wrap=True)

i=1
for row in data:
    row2 = [str(i)] + copy.deepcopy(row)
    t.add_row(*row2)
    i+=1

C.print(t)



import pandas as pd
from openpyxl.utils import get_column_letter
from openpyxl.styles import Font
import subprocess
import os
import pandas
import copy


def finder_expose(filename_or_path):
    subprocess.call("""/bin/bash -c 'open -R "%s"'""" % filename_or_path, shell=True)




def xlsx_sheetnames(filename:str):
    with pandas.ExcelFile(filename) as f:
        return(list(f.sheet_names))

def xlsx_rows_row_callback__r__hdr__row__row_assoc__row_named__context():
    pass

def xlsx_rows(filename:str, sheetname:str, row_callback, context):
    """row_callback(r, hdr, row, row_assoc, row_named, context).
    """
    df = pandas.read_excel(filename, dtype=str, header=None, sheet_name=sheetname, keep_default_na=False)
    df.fillna('')
    n_rows = len(df)
    n_cols = len(df.columns)

    hdr = []
    for col in range(0, n_cols):
        hdr.append(df.loc[0,col])

    # skip row0
    for r in range(1, n_rows):
        row = []
        row_assoc = {}
        row_named = {}
        for col in range(0, n_cols):
            row.append(df.loc[r,col])
            row_assoc[hdr[col]] = df.loc[r,col]
            row_named[get_column_letter(1+col)] = df.loc[r,col]
        row_callback(r+1, hdr, row, row_assoc, row_named, context)


def out(filename:str, sheets:dict):
    xlsx_out(filename=filename, sheets=sheets)


def xlsx_out(filename:str, sheets:dict):
    """Simple Excel Writer.


    mxee.helper.xlsx_out(
        "test2.xlsx",
        sheets={
            'blatt1': {
                'data': [['A', 'B', 'C'],['120']],
                'cw':{'A': 50}
            }
        }
    )
"""
    try:
        with pd.ExcelWriter(filename) as ew:

            for k in sheets.keys():
                dat = sheets[k]['data']

                dfc = pd.DataFrame(dat) # dataframe current
                dfc.to_excel(ew, sheet_name=k, index=False, header=False)
            
                col_widths = {}
                if 'cw' in sheets[k].keys():
                    col_widths = sheets[k]['cw']

                for wdef_k in col_widths.keys():
                    ew.sheets[k].column_dimensions[wdef_k].width = col_widths[wdef_k]

                for header0_idx in range(0, len(dat[0])):
                    column_letter = get_column_letter(header0_idx+1)
                    ew.sheets[k][column_letter + "1"].font = Font(bold=True)
                
                ew.sheets[k].freeze_panes = ew.sheets[k]['A2']

    except Exception as e:
        import os
        os.unlink(filename)
        print(e)


def xlsx_named_columns_callback(r, hdr, row, row_assoc, row_named, ctx):
    for h in hdr:
        if not h in ctx.keys():
            ctx[h] = []

    for h in hdr:
        ctx[h].append(row_assoc[h])

    # sheetname = context['sheet']
    # print(sheetname)
    # for h in hdr:
    #     if context['res']['sheetname']
    #     print(h)
    #     context['h'] = h


def xlsx_named_columns(filename):
    sheetnames = xlsx_sheetnames(filename=filename)
    data = {}
    for sheetname in sheetnames:
        ctx = {}
        xlsx_rows(filename, sheetname, xlsx_named_columns_callback, ctx)
        data[sheetname] = copy.deepcopy(ctx)
    return data


def xlsx_all_columns_by_sheet_and_header(filename):
    """Returns a dict of dicts of lists 'sheetname'->'column-header'->[all str values of column].
    """
    return xlsx_named_columns(filename)


def xlsx(filename):
    """Returns a dict of dicts of lists 'sheetname'->'column-header'->[all str values of column].
    """
    return xlsx_named_columns(filename)


def xlsx0(filename):
    """Returns a dict of lists 'column-header'->[all str values of column] of the first sheet.
    """
    sheetnames = xlsx_sheetnames(filename=filename)
    return xlsx_named_columns(filename)[sheetnames[0]]



def xlsx_all_headers_by_sheet__callback(r,hdr,row,row_assoc,row_named,context):
    if len(context['hdr']) == 0:
        context['hdr'] = copy.deepcopy(hdr)


def headers0(filename):
    sheetnames = xlsx_sheetnames(filename)
    return copy.deepcopy(xlsx_all_headers_by_sheet(filename)[sheetnames[0]])


def xlsx_all_headers_by_sheet(filename):
    sheetnames = xlsx_sheetnames(filename=filename)
    res={}
    for sheetname in sheetnames:
        ctx={'hdr': []}
        xlsx_rows(filename=filename, sheetname=sheetname, row_callback=xlsx_all_headers_by_sheet__callback, context=ctx)
        res[sheetname] = copy.deepcopy(ctx['hdr'])
    return res


def xlsx_column_info_ascii(filename):
    sheetnames = xlsx_sheetnames(filename=filename)
    hdr = xlsx_all_headers_by_sheet(filename=filename)
    lines = []
    for sheetname in sheetnames:
        for col_index in range(0, len(hdr[sheetname])):
            lines.append(sheetname + "\t" + get_column_letter(col_index+1) + "\t" + hdr[sheetname][col_index])
        lines.append("")
    return "\n".join(lines)


def transpose1(src_list):
    res=[]
    for x in src_list:
        res.append([x])
    return res

# def xlsx_all_ascii(filename):
#     """TODO.
#     """
#     sheetnames = xlsx_sheetnames(filename=filename)
#     data = xlsx_all_columns_by_sheet_and_header(filename)
#     res = []
#     for s in sheetnames:
#         sheet_data = data[s]
#     return ""


def sheet_lookup(sheet_data, search_column_name, search_value, target_column_name):
    res = []
    rowcount = len(sheet_data[search_column_name])
    for i in range(0, rowcount):
        if sheet_data[search_column_name][i] == search_value:
            res.append(sheet_data[target_column_name][i])
    return res


def sheet_match(sheet_data, match_column_name, match_values_or, only_headers=None):
    res = []
    rowcount = len(sheet_data[match_column_name])
    for i in range(0, rowcount):
        if sheet_data[match_column_name][i] == match_values_or[0]:
            nrow = []
            for oh in only_headers:
                nrow.append(sheet_data[oh][i])
            res.append(nrow)
    return res


def sheet_flatten(sheet_data, index_column_name, k_name, v_name, fieldnames, row_grep_i=""):
    groups_ = {}
    for x in sheet_data[index_column_name]:
        groups_[x]=True

    groups = list(sorted(groups_.keys()))

    res = [[index_column_name] + copy.deepcopy(fieldnames)]

    stacks = {}
    for f in fieldnames:
        stacks[f] = {}

    for i in range(0, len(sheet_data[index_column_name])):
        g = sheet_data[index_column_name][i]
        k = sheet_data[k_name][i]
        v = sheet_data[v_name][i]
        for f in fieldnames:
            if k == f:
                stacks[k][g] = v

    GREP_I = row_grep_i.upper()

    for g in groups:
        row = [g]
        for f in fieldnames:
            row.append(stacks[f].get(g, ""))
        if row_grep_i == "":
            res.append(row)
        else:
            ROWJOINED = " ".join(row)
            ROWJOINED = ROWJOINED.upper()
            if ROWJOINED.find(GREP_I) >= 0:
                res.append(row)

    return res



def cw_gen(data, colf=1.4):
    col_chars = []
    if len(data) > 0:
        col_chars = [ 0 for x in range(0, len(data[0])) ]
    
    for row in data:
        for rvi in range(0, len(row)):
            new_length = len(str(row[rvi]))
            if new_length > col_chars[rvi]:
                col_chars[rvi] = new_length

    cw={}
    for hci in range(0, len(col_chars)):
        cw[get_column_letter(1+hci)] = int(col_chars[hci] * colf)

    return cw



import sys
import os
import logging

infile = sys.argv[1]
outfile = sys.argv[2]
colchar = "\t"


logging.basicConfig(level=logging.INFO)

logging.info("Reading %s into %s transposed" % (infile, outfile))

rowcount=0
colcount=0

data = None
with open(infile, "r") as f:
    data = [ x.split(colchar) for x in f.read().strip().split("\n")]

rowcount = len(data)
colcount = len(data[0])

#PP(data)
ndata = []

# for row in range(0, rowcount):
#     for col in range(0, colcount):
#         print(row, col)


for col in range(0, colcount):
    ndata.append(list(range(0, rowcount)))


for row in range(0, rowcount):
    for col in range(0, colcount):
        ndata[col][row] = data[row][col]


#print("*"*80)
#PP(ndata)

with open(outfile, 'w') as f:
    for row in ndata:
        f.write(colchar.join(row))
        f.write("\n")



import xler8
import argparse
import copy
from pprint import pprint as PP
import sys


print("usage: $1=textfile with line number line by line, $2=vba outfile")

lines = None
with open(sys.argv[1], 'r') as f:
    lines = [ int(x.strip()) for x in f.read().strip().split("\n") ]

outfile = sys.argv[2]


lines = list(reversed(sorted(lines)))


with open(outfile, 'w') as f:
    f.write("' auto generated xler8.vba row-killer\n");
    f.write("\n");
    f.write("Sub ReverseRowKiller()\n");
    f.write("  With Sheet1\n");
    for i in lines:
        f.write("    .Rows(%d).EntireRow.Delete\n" % i);
    f.write("  End With\n");
    f.write("End Sub\n");
    f.write("\n");

PP(outfile)


import sys

if len(sys.argv) == 1:
    sys.stderr.write("""
usage: PROGRAM col_start_incl col_stop_excl [ divider_string ] [ test ]
hint: columns start an 0 - so for example just regarding the first 4 characters would mean '0 5'
""")
    sys.exit(1)

i_from = int(sys.argv[1])
i_to = int(sys.argv[2])



s_newdivide = ""
try:
    s_newdivide = sys.argv[3]
except:
    pass

s_command = None
try:
    s_command = sys.argv[4]
except:
    pass

segmentation_count = 1

prev_item = None
curr_item = None

prev_line = None
curr_line = None

line_no = 0

for line in sys.stdin.readlines():
    line_no += 1

    curr_line = line
    curr_item = curr_line[i_from:i_to]

    if s_command != None:
        sys.stderr.writelines("(ii) line %d\n" % line_no)
        sys.stderr.writelines("(ii) item = [%s]\n" % curr_item)
        sys.stderr.writelines("(ww) premature test exit, only processed 1 line\n")
        sys.stderr.flush()
        sys.exit(1)

    if prev_item == None:
        prev_line = curr_line
        prev_item = prev_line[i_from:i_to]
        sys.stdout.write(curr_line)
        continue

    if curr_item != prev_item:
        segmentation_count += 1
        sys.stdout.write("%s\n" % s_newdivide)

    sys.stdout.write(curr_line)
    prev_line = curr_line
    prev_item = curr_item

sys.stdout.flush()

sys.stderr.write("(ii) segments in total: %d\n" % segmentation_count)
sys.stderr.flush()
